# Make a Date for Keyboard Maestro

Inserts a date expanded from natural language when `>>date string<` is typed.

Created by Brett Terpstra, 2016, free to use and modify.

### Setup

1. Unzip the archive and open the folder in Finder
1. Move the file `makeadate.js` to a location on your hard drive where it can be referenced
1. Install the `.kmmacro` file in Keyboard Maestro
    - Modify the Execute JavaScript for Automation action, selecting the location of `makeadate.js`
    - Optionally, modify the default date and time format variables (dateFormat, timeFormat)

For the record, this script includes the entire [Sugar.js](https://sugarjs.com) library for handling the natural language. It was too much script to put into the KM editor, so placing it in an external file was the easiest solution.

### Usage

To start converting a date (with the macro enabled), type `>>`, then an optional format shortcut and modifiers, and the natural language version of the date and/or time. Type `<` at the end of the string and Keyboard Maestro will run the script, replacing the entire string (including `>>` and `<`) with the result. If the string is uparseable, the original is returned.

### Natural language

You can type dates in many basic formats. Whether it's absolute (e.g. "Nov 5 3pm") or relative (e.g. "in 20 minutes"), most phrasings will work as long as the language doesn't get too complex.

To create a relative date type `>>tomorrow<` (or just `tom`), or `>>in 3 days<`. Add a time to it using `3pm` or `3:30 pm` (or any permutation). Create an absolute date by using a month and day, optionally a year and time: `>>jul 19 1978<`, or `>>n 7/19/78 6am<` to get "Wednesday, July 19th at 6:00 AM."

#### Increments and shorcuts

When using relative times, you can increment and decrement by typing "in 3 days" or "3 days ago," but you can abbreviate these with m, h, d, y, days, months, years, and min(utes). A number followed by one of these will increment the current time by that amount, and preceding with a minus symbol (`-3d`) will decrement. Note that `3m` is three _minutes_ not _months_. To perform relative date calculation using months, you have to type the word out, e.g. `-3 months`.

### Formats

Common formats:

- `f`: **full** &rarr; *Saturday, November 12, 2016 12:00 AM*
- `l`: **long** &rarr; *November 12, 2016 12:00 AM*
- `m`: **medium** &rarr; *November 12, 2016*
- `n`: **nice (ordinal date, "at" for time)** &rarr; *Saturday, November 12th at 3:00 PM*
- `N`: **nice with year** &rarr; *Sunday, November 13th, 2016 at 3:00 PM*
- `s`: **short** &rarr; *11/12/2016*

Special:

- `d`: **hyphenated date only** &rarr; *2016-11-13*
- `i`: **ISO timestamp** &rarr; *Sat Nov 12 2016 3:00 PM*
- `t`: **time (12hr)** &rarr; *06:30 PM*
- `T`: **time (24hr)** &rarr; *16:30*
- `U`: **unix timestamp (milliseconds)** &rarr; *1478984400000*
- `u`: **unix timestamp** &rarr; _1478984400_
- `x`: **localized date/time string** &rarr; *Sat Nov 12 2016 1:20 PM*

#### Examples

*Examples created on 2016-11-12*

- Today: 
    
    `>>today<` or `>>tod<` &rarr; *2016-11-12*
- Tomorrow: 
    
    `>>tomorrow<` or `>>tom<` &rarr; *2016-11-13*
- Relative date and time:
    
    - `>>5 days<` &rarr; *2016-11-17*
    - `>>5d<` &rarr; *2016-11-17*
    - `>>5 days ago<` &rarr; *2016-11-07*
    - `>>-5d<` &rarr; *2016-11-07*
    - `>>15m<` &rarr; *2016-11-12 12:10pm*
    - `>>15m ago<` &rarr; *2016-11-12 11:40am*
    - `>>-30m<` &rarr; *2016-11-12 11:25am*
- Next wednesday: 
    
    `>>wednesday<` &rarr; *2016-11-16*
- Most recent wednesday: 
    
    `>>- wednesday<` &rarr; *2016-11-09*
    
    _"last" skips to previous week_: `>>last wed<` &rarr; *2016-11-02*
- Absolute date: 
    
    `>>sep 25<` &rarr; *2016-09-25*
    
    `>>sep 25 1965<` &rarr; *1965-09-25*

    `>>sep 25 2017 3pm<` &rarr; *2017-09-25 3:00pm*

- Relative date, absolute time: 
    
    `>>3p tom<` &rarr; *2016-11-13 3:00pm*
- Full date format: 
    
    `>>f sept 25 1965<` &rarr; *Saturday, September 25, 1965 12:00 AM*
- "Nice" format with time: 
    
    `>>n 3p tom<` &rarr; _Sunday, November 13th at 3:00 PM_

- ISO timestamp: 
    
    `>>i now<` &rarr; _Sat Nov 12 2016 1:19 PM_

#### Date format

The format string uses strftime parameters and LDML tokens, which can be combined. <https://sugarjs.com/dates/#/Formatting> for more information.

#### Time format

By default, if no time is specified in natural language, the output date in most formats will not include it. If an input string includes a time, the format specified in the `timeFormat` variable will be appended to the date format. This doesn't apply to `full`, date-only (`d` or `m`),

#### Rounding times

You can include `r` after or instead of a format shortcut to round times to the nearest half hour. If you add a number to it, you can change the amount that it rounds (`r60` to round to the nearest hour). This modifier should come after any format shortcuts: `>>lr30 in 4 hours<`.

Why? Because most of the time when I write "in 2 hours" I don't mean to the minute. At 12:36PM, I wanted "2 hours" to be 2:30PM.

### Past/Future

If a string contains a weekday name or 3-letter abbreviation and does not include the words "last" or "ago", future dates will automatically be forced. Thus typing "wed" will always assume you mean next wed by default. 

You can include a `-` (minus symbol) at the beginning (or immediately after a format option and modifiers), followed by a space to force allow past dates when they're deemed appropriate. So `>>- thursday` would allow it to return the most recent thursday instead of forcing it to find the next one. If using 
a format option and/or modifiers, it must come at the end: `>>lr30- thursday<`.
